import enum
from typing import List
from uuid import UUID

from myst.connectors.source_connector import SourceConnector

HISTORICAL_HOURLY_CONDITIONS_ID = UUID("6f2851e1-7612-4276-a864-172d8c1a060c")


@enum.unique
class Field(str, enum.Enum):
    CLOUD_CEILING = "cloudCeiling"
    CLOUD_COVERAGE_PERCENT = "cloudCoveragePercent"
    EXPIRATION_TIME_UTC = "expirationTimeUtc"
    ICON_CODE = "iconCode"
    ICON_CODE_EXTEND = "iconCodeExtend"
    PRECIP_24_HOUR = "precip24Hour"
    PRESSURE_ALTIMETER = "pressureAltimeter"
    PRESSURE_CHANGE = "pressureChange"
    PRESSURE_MEAN_SEA_LEVEL = "pressureMeanSeaLevel"
    PRESSURE_TENDENCY_CODE = "pressureTendencyCode"
    RELATIVE_HUMIDITY = "relativeHumidity"
    SNOW_24_HOUR = "snow24Hour"
    SUNRISE_TIME_UTC = "sunriseTimeUtc"
    SUNSET_TIME_UTC = "sunsetTimeUtc"
    TEMPERATURE = "temperature"
    TEMPERATURE_CHANGE_24_HOUR = "temperatureChange24Hour"
    TEMPERATURE_DEW_POINT = "temperatureDewPoint"
    TEMPERATURE_FEELS_LIKE = "temperatureFeelsLike"
    TEMPERATURE_HEAT_INDEX = "temperatureHeatIndex"
    TEMPERATURE_MAX_24_HOUR = "temperatureMax24Hour"
    TEMPERATURE_MAX_SINCE_7_AM = "temperatureMaxSince7Am"
    TEMPERATURE_MIN_24_HOUR = "temperatureMin24Hour"
    TEMPERATURE_WIND_CHILL = "temperatureWindChill"
    UV_INDEX = "uvIndex"
    VALID_TIME_UTC = "validTimeUtc"
    VISIBILITY = "visibility"
    WIND_DIRECTION = "windDirection"
    WIND_GUST = "windGust"
    WIND_SPEED = "windSpeed"


class HistoricalHourlyConditions(SourceConnector):
    def __init__(self, latitude: float, longitude: float, fields: List[Field]) -> None:
        super().__init__(
            uuid=HISTORICAL_HOURLY_CONDITIONS_ID, parameters=dict(latitude=latitude, longitude=longitude, fields=fields)
        )



from myst.client import Client
from myst.openapi.models.hpo_get import HPOGet
from myst.openapi.models.hpo_update import HPOUpdate


def request_sync(client: Client, project_uuid: str, uuid: str, json_body: HPOUpdate) -> HPOGet:
    """Updates an HPO in the project."""

    return client.request(
        method="patch", path=f"/projects/{project_uuid}/hpos/{uuid}", response_class=HPOGet, request_model=json_body
    )



from typing import Any, Dict, Optional

from myst.client import Client
from myst.openapi.models.hpo_result_list import HPOResultList


def request_sync(
    client: Client,
    project_uuid: str,
    hpo_uuid: str,
    job_uuid: str,
    include_backtest_result_urls: Optional[bool] = False,
) -> HPOResultList:
    """Filters for HPO results associated with a specific job."""

    params: Dict[str, Any] = {}
    params["job_uuid"] = job_uuid
    params["include_backtest_result_urls"] = include_backtest_result_urls

    params = {k: v for k, v in params.items() if v is not None}

    return client.request(
        method="get",
        path=f"/projects/{project_uuid}/hpos/{hpo_uuid}/results/",
        response_class=HPOResultList,
        params=params,
    )



from myst.client import Client
from myst.openapi.models.backtest_list import BacktestList


def request_sync(client: Client, project_uuid: str) -> BacktestList:
    """Lists backtests."""

    return client.request(method="get", path=f"/projects/{project_uuid}/backtests/", response_class=BacktestList)



from typing import Optional

from pydantic import Field
from typing_extensions import Literal

from myst.models import base_model


class AbsoluteTimingCreate(base_model.BaseModel):
    """Absolute timing schema for create requests."""

    type: Literal["AbsoluteTiming"]
    time: str
    object_: Optional[Literal["Timing"]] = Field("Timing", alias="object")



from typing import TYPE_CHECKING, List, Union
from uuid import UUID

from myst.adapters.utils import get_resource_uuid
from myst.client import get_client
from myst.models.base_model import BaseModel
from myst.models.types import UUIDOrStr, to_uuid
from myst.openapi.api.projects.backtests.jobs import get_backtest_job, list_backtest_jobs
from myst.openapi.models.job_state import JobState

if TYPE_CHECKING:  # Avoid circular imports.
    from myst.resources.backtest import Backtest
    from myst.resources.project import Project


class BacktestJob(BaseModel):

    uuid: UUID
    backtest: UUID
    state: JobState

    @classmethod
    def get(
        cls, project: Union["Project", UUIDOrStr], backtest: Union["Backtest", UUIDOrStr], uuid: UUIDOrStr
    ) -> "BacktestJob":
        """Gets a specific backtest job by its identifier."""
        backtest_job = get_backtest_job.request_sync(
            client=get_client(),
            project_uuid=str(get_resource_uuid(project)),
            backtest_uuid=str(get_resource_uuid(backtest)),
            uuid=str(to_uuid(uuid)),
        )

        return BacktestJob.parse_obj(backtest_job.dict())

    @classmethod
    def list(cls, project: Union["Project", UUIDOrStr], backtest: Union["Backtest", UUIDOrStr]) -> List["BacktestJob"]:
        """Lists all jobs for the backtest."""
        backtest_jobs = list_backtest_jobs.request_sync(
            client=get_client(),
            project_uuid=str(get_resource_uuid(project)),
            backtest_uuid=str(get_resource_uuid(backtest)),
        )

        return [BacktestJob.parse_obj(backtest_job) for backtest_job in backtest_jobs.data]

    def is_completed(self) -> bool:
        """Returns `True` if the job has completed.

        Completed jobs include both successful and failed jobs.

        Returns:
            whether the job has completed
        """
        return self.state in [JobState.SUCCESS, JobState.FAILURE]



from typing import List, Optional

from pydantic import Field
from typing_extensions import Literal

from myst.models import base_model
from myst.openapi.models.deploy_status import DeployStatus
from myst.openapi.models.source_update_parameters import SourceUpdateParameters
from myst.openapi.models.time_dataset_spec import TimeDatasetSpec


class SourceUpdate(base_model.BaseModel):
    """Schema for source update requests."""

    object_: Optional[Literal["Node"]] = Field(..., alias="object")
    uuid: Optional[str] = None
    create_time: Optional[str] = None
    update_time: Optional[str] = None
    organization: Optional[str] = None
    owner: Optional[str] = None
    type: Optional[Literal["Source"]] = None
    title: Optional[str] = None
    description: Optional[str] = None
    project: Optional[str] = None
    deploy_status: Optional[DeployStatus] = None
    connector_uuid: Optional[str] = None
    parameters: Optional[SourceUpdateParameters] = None
    output_specs: Optional[List[TimeDatasetSpec]] = None



import enum
from typing import Optional, Tuple
from uuid import UUID

from myst.connectors.model_connector import ModelConnector


@enum.unique
class GroupName(str, enum.Enum):
    CATEGORICAL_FEATURES = "categorical_features"
    CONTINUOUS_FEATURES = "continuous_features"
    TARGETS = "targets"
    SAMPLE_WEIGHTS = "sample_weights"


@enum.unique
class RegressionLossFunction(str, enum.Enum):
    RMSE = "rmse"
    MAE = "mae"


class MLPRegression(ModelConnector):
    def __init__(
        self,
        batch_size: Optional[int] = None,
        max_training_epochs: Optional[int] = None,
        min_relative_loss_change: Optional[float] = None,
        patience: Optional[int] = None,
        embedding_dimension: Optional[int] = None,
        hidden_layer_dimensions: Optional[Tuple[int, ...]] = None,
        dropout_rate: Optional[float] = None,
        learning_rate: Optional[float] = None,
        weight_decay: Optional[float] = None,
        loss_function: Optional[RegressionLossFunction] = None,
        loss_ceiling: Optional[float] = None,
    ):
        super().__init__(
            uuid=UUID("30c2f4c3-fcb7-4dee-93e6-d6124934ea5e"),
            parameters=dict(
                batch_size=batch_size,
                max_training_epochs=max_training_epochs,
                min_relative_loss_change=min_relative_loss_change,
                patience=patience,
                embedding_dimension=embedding_dimension,
                hidden_layer_dimensions=hidden_layer_dimensions,
                dropout_rate=dropout_rate,
                learning_rate=learning_rate,
                weight_decay=weight_decay,
                loss_function=loss_function,
                loss_ceiling=loss_ceiling,
            ),
        )



from typing import TYPE_CHECKING, Any, Dict, List, Optional, Union

from myst.adapters.utils import drop_unset_values, get_resource_uuid
from myst.client import get_client
from myst.connectors.source_connector import SourceConnector
from myst.models.types import UNSET, OptionalArgument, UUIDOrStr, to_uuid
from myst.openapi.api.projects.sources import create_source, get_source, update_source
from myst.openapi.models.source_create import SourceCreate
from myst.openapi.models.source_update import SourceUpdate
from myst.resources.connector_node import ConnectorNode

if TYPE_CHECKING:  # Avoid circular imports.
    from myst.resources.project import Project


class Source(ConnectorNode):
    """A node which produces data without any inputs."""

    @classmethod
    def create(
        cls,
        project: Union["Project", UUIDOrStr],
        title: str,
        connector: SourceConnector,
        description: Optional[str] = None,
    ) -> "Source":
        """Creates a new source node.

        Args:
            project: the project in which to create the source
            title: the title of the source
            connector: the source connector to use in the source node
            description: a brief description of the source

        Returns:
            the newly created source
        """
        source = create_source.request_sync(
            client=get_client(),
            project_uuid=str(get_resource_uuid(project)),
            json_body=SourceCreate(
                object="Node",
                type="Source",
                title=title,
                description=description,
                connector_uuid=str(connector.uuid),
                parameters=connector.parameters_exclude_none(),
            ),
        )

        return Source.parse_obj(source.dict())

    @classmethod
    def get(cls, project: Union["Project", UUIDOrStr], uuid: UUIDOrStr) -> "Source":
        """Gets a specific source by its identifier."""
        source = get_source.request_sync(
            client=get_client(), project_uuid=str(get_resource_uuid(project)), uuid=str(to_uuid(uuid))
        )

        return Source.parse_obj(source.dict())

    @classmethod
    def list(cls, project: Union["Project", UUIDOrStr]) -> List["Source"]:
        """Gets all sources by project."""
        raise NotImplementedError()

    def update(
        self,
        title: OptionalArgument[str] = UNSET,
        connector: OptionalArgument[SourceConnector] = UNSET,
        description: OptionalArgument[str] = UNSET,
    ) -> "Source":
        """Updates source."""
        parameters = connector.parameters_exclude_none() if isinstance(connector, SourceConnector) else UNSET
        source_update: Dict[str, Any] = dict(
            object="Node",
            type="Source",
            title=title,
            connector_uuid=str(connector.uuid) if isinstance(connector, SourceConnector) else UNSET,
            parameters=parameters,
            description=description,
        )

        source = update_source.request_sync(
            client=get_client(),
            project_uuid=str(self.project),
            uuid=str(self.uuid),
            json_body=SourceUpdate.parse_obj(drop_unset_values(source_update)),
        )

        return Source.parse_obj(source.dict())



from typing import Any, Dict
from uuid import UUID

from myst.resources.node import Node


class ConnectorNode(Node):
    """A node backed by a connector.

    Attributes:
        connector_uuid: the UUID of the connector underlying this node
        parameters: the parameters to apply to the connector underlying this node
    """

    connector_uuid: UUID
    parameters: Dict[str, Any]



try:
    import importlib.metadata as importlib_metadata
except ModuleNotFoundError:
    # For Python <3.8 add 'importlib_metadata' as a dependency.
    import importlib_metadata  # type: ignore


# NOTE: Must match the name in `pyproject.toml`!
PACKAGE_NAME = "myst-alpha"


def get_package_version() -> str:
    """Returns the package version."""
    # I promise you, this package really does have the attribute "version".
    return importlib_metadata.version(PACKAGE_NAME)  # type: ignore



from myst.connectors.operation_connectors.cauchy_distribution import CauchyDistribution
from myst.connectors.operation_connectors.fill_na import FillNA
from myst.connectors.operation_connectors.get_dummies import GetDummies
from myst.connectors.operation_connectors.laplace_distribution import LaplaceDistribution
from myst.connectors.operation_connectors.lognormal_distribution import LogNormalDistribution
from myst.connectors.operation_connectors.normal_distribution import NormalDistribution
from myst.connectors.operation_connectors.numerical_expression import NumericalExpression
from myst.connectors.operation_connectors.resampling import Resampling
from myst.connectors.operation_connectors.time_transformations import TimeTransformations

__all__ = [
    "FillNA",
    "GetDummies",
    "CauchyDistribution",
    "LaplaceDistribution",
    "LogNormalDistribution",
    "NormalDistribution",
    "NumericalExpression",
    "Resampling",
    "TimeTransformations",
]



from typing import List

from myst.models import base_model
from myst.openapi.models.time_series_run_result_list_item import TimeSeriesRunResultListItem


class TimeSeriesRunResultList(base_model.BaseModel):
    """Schema for time series run result list responses."""

    data: List[TimeSeriesRunResultListItem]
    has_more: bool



import enum
from typing import List
from uuid import UUID

from myst.connectors.source_connector import SourceConnector

ENHANCED_FORECAST_ID = UUID("1aeaadec-6912-422a-a4fc-5d6765391c17")


@enum.unique
class Field(str, enum.Enum):
    CLOUD_COVER = "cloudCover"
    PRECIP_CHANCE = "precipChance"
    PRECIP_TYPE = "precipType"
    PRESSURE_MEAN_SEA_LEVEL = "pressureMeanSeaLevel"
    QPF = "qpf"
    QPF_SNOW = "qpfSnow"
    RELATIVE_HUMIDITY = "relativeHumidity"
    TEMPERATURE = "temperature"
    TEMPERATURE_DEW_POINT = "temperatureDewPoint"
    TEMPERATURE_FEELS_LIKE = "temperatureFeelsLike"
    TEMPERATURE_HEAT_INDEX = "temperatureHeatIndex"
    TEMPERATURE_WIND_CHILL = "temperatureWindChill"
    UV_INDEX = "uvIndex"
    VISIBILITY = "visibility"
    WIND_DIRECTION = "windDirection"
    WIND_GUST = "windGust"
    WIND_SPEED = "windSpeed"


class EnhancedForecast(SourceConnector):
    def __init__(self, latitude: float, longitude: float, fields: List[Field]) -> None:
        super().__init__(
            uuid=ENHANCED_FORECAST_ID, parameters=dict(latitude=latitude, longitude=longitude, fields=fields)
        )



from typing import Optional

from pydantic import Field
from typing_extensions import Literal

from myst.models import base_model


class DeploymentCreate(base_model.BaseModel):
    """Schema for deployment create requests."""

    title: str
    object_: Optional[Literal["Deployment"]] = Field("Deployment", alias="object")



from myst.client import Client
from myst.openapi.models.user_get import UserGet
from myst.openapi.models.user_update import UserUpdate


def request_sync(client: Client, uuid: str, json_body: UserUpdate) -> UserGet:
    """Updates a user."""

    return client.request(method="patch", path=f"/users/{uuid}", response_class=UserGet, request_model=json_body)
